"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GeometryCodes = void 0;
var simple_features_js_1 = require("@ngageoint/simple-features-js");
/**
 * Geometry Code utilities to convert between geometry attributes and geometry codes
 */
var GeometryCodes = /** @class */ (function () {
    function GeometryCodes() {
    }
    /**
     * Get the geometry code from the geometry
     * @param geometry geometry
     * @return geometry code
     */
    GeometryCodes.getCode = function (geometry) {
        return GeometryCodes._getCode(geometry.geometryType, geometry.hasZ, geometry.hasM);
    };
    /**
     * Get the geometry code from the geometry type
     * @param geometryType geometry type
     * @param hasZ has z
     * @param hasM mas m
     * @return geometry code
     * @since 2.0.3
     */
    GeometryCodes._getCode = function (geometryType, hasZ, hasM) {
        var code = GeometryCodes.getCodeForGeometryType(geometryType);
        if (hasZ) {
            code += 1000;
        }
        if (hasM) {
            code += 2000;
        }
        return code;
    };
    /**
     * Get the geometry code from the geometry type
     *
     * @param geometryType geometry type
     * @return geometry code
     */
    GeometryCodes.getCodeForGeometryType = function (geometryType) {
        var code;
        switch (geometryType) {
            case simple_features_js_1.GeometryType.GEOMETRY:
                code = 0;
                break;
            case simple_features_js_1.GeometryType.POINT:
                code = 1;
                break;
            case simple_features_js_1.GeometryType.LINESTRING:
                code = 2;
                break;
            case simple_features_js_1.GeometryType.POLYGON:
                code = 3;
                break;
            case simple_features_js_1.GeometryType.MULTIPOINT:
                code = 4;
                break;
            case simple_features_js_1.GeometryType.MULTILINESTRING:
                code = 5;
                break;
            case simple_features_js_1.GeometryType.MULTIPOLYGON:
                code = 6;
                break;
            case simple_features_js_1.GeometryType.GEOMETRYCOLLECTION:
                code = 7;
                break;
            case simple_features_js_1.GeometryType.CIRCULARSTRING:
                code = 8;
                break;
            case simple_features_js_1.GeometryType.COMPOUNDCURVE:
                code = 9;
                break;
            case simple_features_js_1.GeometryType.CURVEPOLYGON:
                code = 10;
                break;
            case simple_features_js_1.GeometryType.MULTICURVE:
                code = 11;
                break;
            case simple_features_js_1.GeometryType.MULTISURFACE:
                code = 12;
                break;
            case simple_features_js_1.GeometryType.CURVE:
                code = 13;
                break;
            case simple_features_js_1.GeometryType.SURFACE:
                code = 14;
                break;
            case simple_features_js_1.GeometryType.POLYHEDRALSURFACE:
                code = 15;
                break;
            case simple_features_js_1.GeometryType.TIN:
                code = 16;
                break;
            case simple_features_js_1.GeometryType.TRIANGLE:
                code = 17;
                break;
            default:
                throw new simple_features_js_1.SFException("Unsupported Geometry Type for code retrieval: " + geometryType);
        }
        return code;
    };
    /**
     * Get the Geometry Type from the code
     * @param code geometry type code
     * @return geometry type
     */
    GeometryCodes.getGeometryType = function (code) {
        // Look at the last 2 digits to find the geometry type code
        var geometryTypeCode = code % 1000;
        var geometryType = null;
        switch (geometryTypeCode) {
            case 0:
                geometryType = simple_features_js_1.GeometryType.GEOMETRY;
                break;
            case 1:
                geometryType = simple_features_js_1.GeometryType.POINT;
                break;
            case 2:
                geometryType = simple_features_js_1.GeometryType.LINESTRING;
                break;
            case 3:
                geometryType = simple_features_js_1.GeometryType.POLYGON;
                break;
            case 4:
                geometryType = simple_features_js_1.GeometryType.MULTIPOINT;
                break;
            case 5:
                geometryType = simple_features_js_1.GeometryType.MULTILINESTRING;
                break;
            case 6:
                geometryType = simple_features_js_1.GeometryType.MULTIPOLYGON;
                break;
            case 7:
                geometryType = simple_features_js_1.GeometryType.GEOMETRYCOLLECTION;
                break;
            case 8:
                geometryType = simple_features_js_1.GeometryType.CIRCULARSTRING;
                break;
            case 9:
                geometryType = simple_features_js_1.GeometryType.COMPOUNDCURVE;
                break;
            case 10:
                geometryType = simple_features_js_1.GeometryType.CURVEPOLYGON;
                break;
            case 11:
                geometryType = simple_features_js_1.GeometryType.MULTICURVE;
                break;
            case 12:
                geometryType = simple_features_js_1.GeometryType.MULTISURFACE;
                break;
            case 13:
                geometryType = simple_features_js_1.GeometryType.CURVE;
                break;
            case 14:
                geometryType = simple_features_js_1.GeometryType.SURFACE;
                break;
            case 15:
                geometryType = simple_features_js_1.GeometryType.POLYHEDRALSURFACE;
                break;
            case 16:
                geometryType = simple_features_js_1.GeometryType.TIN;
                break;
            case 17:
                geometryType = simple_features_js_1.GeometryType.TRIANGLE;
                break;
            default:
                throw new simple_features_js_1.SFException("Unsupported Geometry code for type retrieval: " + code);
        }
        return geometryType;
    };
    /**
     * Determine if the geometry code has a Z (3D) value
     *
     * @param code
     *            geometry code
     * @return true is has Z
     */
    GeometryCodes.hasZ = function (code) {
        var hasZ = false;
        var mode = GeometryCodes.getGeometryMode(code);
        switch (mode) {
            case 0:
            case 2:
                break;
            case 1:
            case 3:
                hasZ = true;
                break;
            default:
                throw new simple_features_js_1.SFException("Unexpected Geometry code for Z determination: " + code);
        }
        return hasZ;
    };
    /**
     * Determine if the geometry code has a M (linear referencing system) value
     * @param code geometry code
     * @return true is has M
     */
    GeometryCodes.hasM = function (code) {
        var hasM = false;
        var mode = GeometryCodes.getGeometryMode(code);
        switch (mode) {
            case 0:
            case 1:
                break;
            case 2:
            case 3:
                hasM = true;
                break;
            default:
                throw new simple_features_js_1.SFException("Unexpected Geometry code for M determination: " + code);
        }
        return hasM;
    };
    /**
     * Get the geometry mode from the geometry code. Returns the digit in the
     * thousands place. (z is enabled when 1 or 3, m is enabled when 2 or 3)
     *
     * @param code geometry code
     * @return geometry mode
     */
    GeometryCodes.getGeometryMode = function (code) {
        return Math.round(code / 1000);
    };
    return GeometryCodes;
}());
exports.GeometryCodes = GeometryCodes;
//# sourceMappingURL=GeometryCodes.js.map