"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GeometryReader = void 0;
var simple_features_js_1 = require("@ngageoint/simple-features-js");
var ByteReader_1 = require("./ByteReader");
var GeometryTypeInfo_1 = require("./GeometryTypeInfo");
var ByteOrder_1 = require("./ByteOrder");
var GeometryCodes_1 = require("./GeometryCodes");
/**
 * Well Known Binary reader
 */
var GeometryReader = /** @class */ (function () {
    /**
     * Constructor
     * @param args
     */
    function GeometryReader() {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        if (args.length === 1 && args[0] instanceof ByteReader_1.ByteReader) {
            this.reader = args[0];
        }
        else if (args.length === 1 && args[0] instanceof Buffer) {
            this.reader = new ByteReader_1.ByteReader(args[0]);
        }
    }
    /**
     * Get the byte reader
     * @return byte reader
     */
    GeometryReader.prototype.getByteReader = function () {
        return this.reader;
    };
    /**
     * Read a geometry from the byte reader
     *
     * @param filter geometry filter
     * @param containingType containing geometry type
     * @return geometry
     */
    GeometryReader.prototype.read = function (filter, containingType) {
        if (containingType === void 0) { containingType = undefined; }
        var originalByteOrder = this.reader.getByteOrder();
        // Read the byte order and geometry type
        var geometryTypeInfo = this.readGeometryType();
        var geometryType = geometryTypeInfo.geometryType;
        var hasZ = geometryTypeInfo.hasZ;
        var hasM = geometryTypeInfo.hasM;
        var geometry = null;
        switch (geometryType) {
            case simple_features_js_1.GeometryType.GEOMETRY:
                throw new simple_features_js_1.SFException("Unexpected Geometry Type of Geometry which is abstract");
            case simple_features_js_1.GeometryType.POINT:
                geometry = this.readPoint(hasZ, hasM);
                break;
            case simple_features_js_1.GeometryType.LINESTRING:
                geometry = this.readLineString(filter, hasZ, hasM);
                break;
            case simple_features_js_1.GeometryType.POLYGON:
                geometry = this.readPolygon(filter, hasZ, hasM);
                break;
            case simple_features_js_1.GeometryType.MULTIPOINT:
                geometry = this.readMultiPoint(filter, hasZ, hasM);
                break;
            case simple_features_js_1.GeometryType.MULTILINESTRING:
                geometry = this.readMultiLineString(filter, hasZ, hasM);
                break;
            case simple_features_js_1.GeometryType.MULTIPOLYGON:
                geometry = this.readMultiPolygon(filter, hasZ, hasM);
                break;
            case simple_features_js_1.GeometryType.GEOMETRYCOLLECTION:
            case simple_features_js_1.GeometryType.MULTICURVE:
            case simple_features_js_1.GeometryType.MULTISURFACE:
                geometry = this.readGeometryCollection(filter, hasZ, hasM);
                break;
            case simple_features_js_1.GeometryType.CIRCULARSTRING:
                geometry = this.readCircularString(filter, hasZ, hasM);
                break;
            case simple_features_js_1.GeometryType.COMPOUNDCURVE:
                geometry = this.readCompoundCurve(filter, hasZ, hasM);
                break;
            case simple_features_js_1.GeometryType.CURVEPOLYGON:
                geometry = this.readCurvePolygon(filter, hasZ, hasM);
                break;
            case simple_features_js_1.GeometryType.CURVE:
                throw new simple_features_js_1.SFException("Unexpected Geometry Type of Curve which is abstract");
            case simple_features_js_1.GeometryType.SURFACE:
                throw new simple_features_js_1.SFException("Unexpected Geometry Type of Surface which is abstract");
            case simple_features_js_1.GeometryType.POLYHEDRALSURFACE:
                geometry = this.readPolyhedralSurface(filter, hasZ, hasM);
                break;
            case simple_features_js_1.GeometryType.TIN:
                geometry = this.readTIN(filter, hasZ, hasM);
                break;
            case simple_features_js_1.GeometryType.TRIANGLE:
                geometry = this.readTriangle(filter, hasZ, hasM);
                break;
            default:
                throw new simple_features_js_1.SFException("Geometry Type not supported: " + geometryType);
        }
        if (!GeometryReader.filter(filter, containingType, geometry)) {
            geometry = null;
        }
        // Restore the byte order
        this.reader.setByteOrder(originalByteOrder);
        return geometry;
    };
    /**
     * Read the geometry type info
     * @return geometry type info
     */
    GeometryReader.prototype.readGeometryType = function () {
        // Read the single byte order byte
        var byteOrderValue = this.reader.readByte();
        var byteOrder = byteOrderValue === 0 ? ByteOrder_1.ByteOrder.BIG_ENDIAN : ByteOrder_1.ByteOrder.LITTLE_ENDIAN;
        this.reader.setByteOrder(byteOrder);
        // Read the geometry type unsigned integer
        var unsignedGeometryTypeCode = this.reader.readUInt32();
        // Check for 2.5D geometry types
        var hasZ = false;
        if (unsignedGeometryTypeCode > GeometryReader.WKB25D) {
            hasZ = true;
            unsignedGeometryTypeCode -= GeometryReader.WKB25D;
        }
        var geometryTypeCode = unsignedGeometryTypeCode;
        // Determine the geometry type
        var geometryType = GeometryCodes_1.GeometryCodes.getGeometryType(geometryTypeCode);
        // Determine if the geometry has a z (3d) or m (linear referencing system) value
        if (!hasZ) {
            hasZ = GeometryCodes_1.GeometryCodes.hasZ(geometryTypeCode);
        }
        var hasM = GeometryCodes_1.GeometryCodes.hasM(geometryTypeCode);
        return new GeometryTypeInfo_1.GeometryTypeInfo(geometryTypeCode, geometryType, hasZ, hasM);
    };
    /**
     * Read a Point
     * @param hasZ has z flag
     * @param hasM has m flag
     * @return point
     */
    GeometryReader.prototype.readPoint = function (hasZ, hasM) {
        var x = this.reader.readDouble();
        var y = this.reader.readDouble();
        var point = new simple_features_js_1.Point(hasZ, hasM, x, y);
        if (hasZ) {
            point.z = this.reader.readDouble();
        }
        if (hasM) {
            point.m = this.reader.readDouble();
        }
        return point;
    };
    /**
     * Read a Line String
     * @param filter geometry filter
     * @param hasZ has z flag
     * @param hasM has m flag
     * @return line string
     */
    GeometryReader.prototype.readLineString = function (filter, hasZ, hasM) {
        var lineString = new simple_features_js_1.LineString(hasZ, hasM);
        var numPoints = this.reader.readInt();
        for (var i = 0; i < numPoints; i++) {
            var point = this.readPoint(hasZ, hasM);
            if (GeometryReader.filter(filter, simple_features_js_1.GeometryType.LINESTRING, point)) {
                lineString.addPoint(point);
            }
        }
        return lineString;
    };
    /**
     * Read a Polygon
     * @param filter geometry filter
     * @param hasZ  has z flag
     * @param hasM has m flag
     * @return polygon
     */
    GeometryReader.prototype.readPolygon = function (filter, hasZ, hasM) {
        var polygon = new simple_features_js_1.Polygon(hasZ, hasM);
        var numRings = this.reader.readInt();
        for (var i = 0; i < numRings; i++) {
            var ring = this.readLineString(filter, hasZ, hasM);
            if (GeometryReader.filter(filter, simple_features_js_1.GeometryType.POLYGON, ring)) {
                polygon.addRing(ring);
            }
        }
        return polygon;
    };
    /**
     * Read a Multi Point
     * @param filter geometry filter
     * @param hasZ has z flag
     * @param hasM has m flag
     * @return multi point
     */
    GeometryReader.prototype.readMultiPoint = function (filter, hasZ, hasM) {
        var multiPoint = new simple_features_js_1.MultiPoint(hasZ, hasM);
        var numPoints = this.reader.readInt();
        for (var i = 0; i < numPoints; i++) {
            var point = this.read(filter, simple_features_js_1.GeometryType.MULTIPOINT);
            if (point != null) {
                multiPoint.addPoint(point);
            }
        }
        return multiPoint;
    };
    /**
     * Read a Multi Line String
     * @param filter geometry filter
     * @param hasZ has z flag
     * @param hasM has m flag
     * @return multi line string
     */
    GeometryReader.prototype.readMultiLineString = function (filter, hasZ, hasM) {
        var multiLineString = new simple_features_js_1.MultiLineString(hasZ, hasM);
        var numLineStrings = this.reader.readInt();
        for (var i = 0; i < numLineStrings; i++) {
            var lineString = this.read(filter, simple_features_js_1.GeometryType.MULTILINESTRING);
            if (lineString != null) {
                multiLineString.addLineString(lineString);
            }
        }
        return multiLineString;
    };
    /**
     * Read a Multi Polygon
     * @param filter geometry filter
     * @param hasZ has z flag
     * @param hasM has m flag
     * @return multi polygon
     */
    GeometryReader.prototype.readMultiPolygon = function (filter, hasZ, hasM) {
        var multiPolygon = new simple_features_js_1.MultiPolygon(hasZ, hasM);
        var numPolygons = this.reader.readInt();
        for (var i = 0; i < numPolygons; i++) {
            var polygon = this.read(filter, simple_features_js_1.GeometryType.MULTIPOLYGON);
            if (polygon != null) {
                multiPolygon.addPolygon(polygon);
            }
        }
        return multiPolygon;
    };
    /**
     * Read a Geometry Collection
     * @param filter geometry filter
     * @param hasZ has z flag
     * @param hasM has m flag
     * @return geometry collection
     */
    GeometryReader.prototype.readGeometryCollection = function (filter, hasZ, hasM) {
        var geometryCollection = new simple_features_js_1.GeometryCollection(hasZ, hasM);
        var numGeometries = this.reader.readInt();
        for (var i = 0; i < numGeometries; i++) {
            var geometry = this.read(filter, simple_features_js_1.GeometryType.GEOMETRYCOLLECTION);
            if (geometry != null) {
                geometryCollection.addGeometry(geometry);
            }
        }
        return geometryCollection;
    };
    /**
     * Read a Circular String
     * @param filter geometry filter
     * @param hasZ has z flag
     * @param hasM has m flag
     * @return circular string
     */
    GeometryReader.prototype.readCircularString = function (filter, hasZ, hasM) {
        var circularString = new simple_features_js_1.CircularString(hasZ, hasM);
        var numPoints = this.reader.readInt();
        for (var i = 0; i < numPoints; i++) {
            var point = this.readPoint(hasZ, hasM);
            if (GeometryReader.filter(filter, simple_features_js_1.GeometryType.CIRCULARSTRING, point)) {
                circularString.addPoint(point);
            }
        }
        return circularString;
    };
    /**
     * Read a Compound Curve
     * @param filter geometry filter
     * @param hasZ has z flag
     * @param hasM has m flag
     * @return compound curve
     */
    GeometryReader.prototype.readCompoundCurve = function (filter, hasZ, hasM) {
        var compoundCurve = new simple_features_js_1.CompoundCurve(hasZ, hasM);
        var numLineStrings = this.reader.readInt();
        for (var i = 0; i < numLineStrings; i++) {
            var lineString = this.read(filter, simple_features_js_1.GeometryType.COMPOUNDCURVE);
            if (lineString != null) {
                compoundCurve.addLineString(lineString);
            }
        }
        return compoundCurve;
    };
    /**
     * Read a Curve Polygon
     * @param filter geometry filter
     * @param hasZ has z flag
     * @param hasM has m flag
     * @return curve polygon
     */
    GeometryReader.prototype.readCurvePolygon = function (filter, hasZ, hasM) {
        var curvePolygon = new simple_features_js_1.CurvePolygon(hasZ, hasM);
        var numRings = this.reader.readInt();
        for (var i = 0; i < numRings; i++) {
            var ring = this.read(filter, simple_features_js_1.GeometryType.CURVEPOLYGON);
            if (ring != null) {
                curvePolygon.addRing(ring);
            }
        }
        return curvePolygon;
    };
    /**
     * Read a Polyhedral Surface
     * @param filter geometry filter
     * @param hasZ has z flag
     * @param hasM has m flag
     * @return polyhedral surface
     */
    GeometryReader.prototype.readPolyhedralSurface = function (filter, hasZ, hasM) {
        var polyhedralSurface = new simple_features_js_1.PolyhedralSurface(hasZ, hasM);
        var numPolygons = this.reader.readInt();
        for (var i = 0; i < numPolygons; i++) {
            var polygon = this.read(filter, simple_features_js_1.GeometryType.POLYHEDRALSURFACE);
            if (polygon != null) {
                polyhedralSurface.addPolygon(polygon);
            }
        }
        return polyhedralSurface;
    };
    /**
     * Read a TIN
     * @param filter geometry filter
     * @param hasZ has z flag
     * @param hasM  has m flag
     * @return TIN
     */
    GeometryReader.prototype.readTIN = function (filter, hasZ, hasM) {
        var tin = new simple_features_js_1.TIN(hasZ, hasM);
        var numPolygons = this.reader.readInt();
        for (var i = 0; i < numPolygons; i++) {
            var polygon = this.read(filter, simple_features_js_1.GeometryType.TIN);
            if (polygon != null) {
                tin.addPolygon(polygon);
            }
        }
        return tin;
    };
    /**
     * Read a Triangle
     * @param filter geometry filter
     * @param hasZ has z flag
     * @param hasM has m flag
     * @return triangle
     */
    GeometryReader.prototype.readTriangle = function (filter, hasZ, hasM) {
        var triangle = new simple_features_js_1.Triangle(hasZ, hasM);
        var numRings = this.reader.readInt();
        for (var i = 0; i < numRings; i++) {
            var ring = this.readLineString(filter, hasZ, hasM);
            if (GeometryReader.filter(filter, simple_features_js_1.GeometryType.TRIANGLE, ring)) {
                triangle.addRing(ring);
            }
        }
        return triangle;
    };
    /**
     * Read a geometry from the byte reader
     * @param buffer buffer
     * @param filter geometry filter
     * @param containingType containing geometry type
     * @return geometry
     */
    GeometryReader.readGeometry = function (buffer, filter, containingType) {
        if (filter === void 0) { filter = undefined; }
        if (containingType === void 0) { containingType = undefined; }
        var geometryReader = new GeometryReader(buffer);
        return geometryReader.read(filter, containingType);
    };
    /**
     * Read a geometry from the byte reader
     * @param reader byte reader
     * @param filter geometry filter
     * @param containingType containing geometry type
     * @return geometry
     */
    GeometryReader.readGeometryWithByteReader = function (reader, filter, containingType) {
        if (filter === void 0) { filter = undefined; }
        if (containingType === void 0) { containingType = undefined; }
        var geometryReader = new GeometryReader(reader);
        return geometryReader.read(filter, containingType);
    };
    /**
     * Read the geometry type info
     * @param reader byte reader
     * @return geometry type info
     */
    GeometryReader.readGeometryType = function (reader) {
        var geometryReader = new GeometryReader(reader);
        return geometryReader.readGeometryType();
    };
    /**
     * Read a Point
     * @param reader byte reader
     * @param hasZ has z flag
     * @param hasM has m flag
     * @return point
     */
    GeometryReader.readPoint = function (reader, hasZ, hasM) {
        var geometryReader = new GeometryReader(reader);
        return geometryReader.readPoint(hasZ, hasM);
    };
    /**
     * Read a Line String
     * @param reader byte reader
     * @param filter geometry filter
     * @param hasZ has z flag
     * @param hasM has m flag
     * @return line string
     */
    GeometryReader.readLineString = function (reader, filter, hasZ, hasM) {
        var geometryReader = new GeometryReader(reader);
        return geometryReader.readLineString(filter, hasZ, hasM);
    };
    /**
     * Read a Polygon
     * @param reader byte reader
     * @param filter geometry filter
     * @param hasZ has z flag
     * @param hasM has m flag
     * @return polygon
     */
    GeometryReader.readPolygon = function (reader, filter, hasZ, hasM) {
        var geometryReader = new GeometryReader(reader);
        return geometryReader.readPolygon(filter, hasZ, hasM);
    };
    /**
     * Read a Multi Point
     * @param reader byte reader
     * @param filter geometry filter
     * @param hasZ has z flag
     * @param hasM has m flag
     * @return multi point
     */
    GeometryReader.readMultiPoint = function (reader, filter, hasZ, hasM) {
        var geometryReader = new GeometryReader(reader);
        return geometryReader.readMultiPoint(filter, hasZ, hasM);
    };
    /**
     * Read a Multi Line String
     * @param reader byte reader
     * @param filter geometry filter
     * @param hasZ has z flag
     * @param hasM has m flag
     * @return multi line string
     */
    GeometryReader.readMultiLineString = function (reader, filter, hasZ, hasM) {
        var geometryReader = new GeometryReader(reader);
        return geometryReader.readMultiLineString(filter, hasZ, hasM);
    };
    /**
     * Read a Multi Polygon
     * @param reader byte reader
     * @param filter geometry filter
     * @param hasZ has z flag
     * @param hasM has m flag
     * @return multi polygon
     */
    GeometryReader.readMultiPolygon = function (reader, filter, hasZ, hasM) {
        var geometryReader = new GeometryReader(reader);
        return geometryReader.readMultiPolygon(filter, hasZ, hasM);
    };
    /**
     * Read a Geometry Collection
     * @param reader byte reader
     * @param filter geometry filter
     * @param hasZ has z flag
     * @param hasM has m flag
     * @return geometry collection
     */
    GeometryReader.readGeometryCollection = function (reader, filter, hasZ, hasM) {
        var geometryReader = new GeometryReader(reader);
        return geometryReader.readGeometryCollection(filter, hasZ, hasM);
    };
    /**
     * Read a Circular String
     * @param reader byte reader
     * @param filter geometry filter
     * @param hasZ has z flag
     * @param hasM has m flag
     * @return circular string
     */
    GeometryReader.readCircularString = function (reader, filter, hasZ, hasM) {
        var geometryReader = new GeometryReader(reader);
        return geometryReader.readCircularString(filter, hasZ, hasM);
    };
    /**
     * Read a Compound Curve
     * @param reader byte reader
     * @param filter geometry filter
     * @param hasZ has z flag
     * @param hasM has m flag
     * @return compound curve
     */
    GeometryReader.readCompoundCurve = function (reader, filter, hasZ, hasM) {
        var geometryReader = new GeometryReader(reader);
        return geometryReader.readCompoundCurve(filter, hasZ, hasM);
    };
    /**
     * Read a Curve Polygon
     * @param reader byte reader
     * @param filter geometry filter
     * @param hasZ has z flag
     * @param hasM has m flag
     * @return curve polygon
     */
    GeometryReader.readCurvePolygon = function (reader, filter, hasZ, hasM) {
        var geometryReader = new GeometryReader(reader);
        return geometryReader.readCurvePolygon(filter, hasZ, hasM);
    };
    /**
     * Read a Polyhedral Surface
     * @param reader byte reader
     * @param filter geometry filter
     * @param hasZ has z flag
     * @param hasM has m flag
     * @return polyhedral surface
     */
    GeometryReader.readPolyhedralSurface = function (reader, filter, hasZ, hasM) {
        var geometryReader = new GeometryReader(reader);
        return geometryReader.readPolyhedralSurface(filter, hasZ, hasM);
    };
    /**
     * Read a TIN
     * @param reader byte reader
     * @param filter geometry filter
     * @param hasZ has z flag
     * @param hasM has m flag
     * @return TIN
     */
    GeometryReader.readTIN = function (reader, filter, hasZ, hasM) {
        var geometryReader = new GeometryReader(reader);
        return geometryReader.readTIN(filter, hasZ, hasM);
    };
    /**
     * Read a Triangle
     * @param reader byte reader
     * @param filter geometry filter
     * @param hasZ has z flag
     * @param hasM has m flag
     * @return triangle
     */
    GeometryReader.readTriangle = function (reader, filter, hasZ, hasM) {
        var geometryReader = new GeometryReader(reader);
        return geometryReader.readTriangle(filter, hasZ, hasM);
    };
    /**
     * Filter the geometry
     *
     * @param filter geometry filter or null
     * @param containingType containing geometry type
     * @param geometry geometry or null
     * @return true if passes filter
     */
    GeometryReader.filter = function (filter, containingType, geometry) {
        return filter == null || geometry == null || filter.filter(containingType, geometry);
    };
    /**
     * 2.5D bit
     */
    GeometryReader.WKB25D = 0x80000000;
    return GeometryReader;
}());
exports.GeometryReader = GeometryReader;
//# sourceMappingURL=GeometryReader.js.map